@@ -20,9 +20,14 @@
 #include "COpenGLExtensionHandler.h"
 
 #include "guiengine/engine.hpp"
+#include "ge_main.hpp"
 #include "glad/gl.h"
+#include "ge_vulkan_driver.hpp"
+#include "ge_vulkan_scene_manager.hpp"
 #include "MoltenVK.h"
 
+#include <SDL_vulkan.h>
+
 extern bool GLContextDebugBit;
 
 namespace irr
@@ -40,7 +45,7 @@ namespace irr
 #endif
 #ifdef _IRR_COMPILE_WITH_VULKAN_
 		IVideoDriver* createVulkanDriver(const SIrrlichtCreationParameters& params,
-			io::IFileSystem* io, SDL_Window* win);
+			io::IFileSystem* io, SDL_Window* win, IrrlichtDevice* device);
 #endif
 	} // end namespace video
 
@@ -53,10 +58,6 @@ extern "C" int Android_disablePadding();
 
 namespace irr
 {
-
-float g_native_scale_x = 1.0f;
-float g_native_scale_y = 1.0f;
-
 //! constructor
 CIrrDeviceSDL::CIrrDeviceSDL(const SIrrlichtCreationParameters& param)
 	: CIrrDeviceStub(param),
@@ -66,7 +67,8 @@ CIrrDeviceSDL::CIrrDeviceSDL(const SIrrlichtCreationParameters& param)
 	TopPadding(0), BottomPadding(0), LeftPadding(0), RightPadding(0),
 	InitialOrientation(0), WindowHasFocus(false), WindowMinimized(false),
 	Resizable(false), AccelerometerIndex(-1), AccelerometerInstance(-1),
-	GyroscopeIndex(-1), GyroscopeInstance(-1)
+	GyroscopeIndex(-1), GyroscopeInstance(-1), NativeScaleX(1.0f),
+	NativeScaleY(1.0f)
 {
 	#ifdef _DEBUG
 	setDebugName("CIrrDeviceSDL");
@@ -81,6 +83,9 @@ CIrrDeviceSDL::CIrrDeviceSDL(const SIrrlichtCreationParameters& param)
 	// noparachute prevents SDL from catching fatal errors.
 	SDL_SetHint(SDL_HINT_NO_SIGNAL_HANDLERS, "1");
 	SDL_SetHint(SDL_HINT_ACCELEROMETER_AS_JOYSTICK, "0");
+#if SDL_VERSION_ATLEAST(2, 0, 18)
+	SDL_SetHint(SDL_HINT_IME_SHOW_UI, "1");
+#endif
 
 	// Switch SDL disables this hint by default: https://github.com/devkitPro/SDL/pull/55#issuecomment-633775255
 	SDL_SetHint(SDL_HINT_TOUCH_MOUSE_EVENTS, "1");
@@ -88,7 +93,12 @@ CIrrDeviceSDL::CIrrDeviceSDL(const SIrrlichtCreationParameters& param)
 #ifndef MOBILE_STK
 	// Prevent fullscreen minimizes when losing focus
 	if (CreationParams.Fullscreen)
-		SDL_SetHint(SDL_HINT_VIDEO_MINIMIZE_ON_FOCUS_LOSS, "0");
+	{
+		if (CreationParams.DriverType != video::EDT_VULKAN)
+			SDL_SetHint(SDL_HINT_VIDEO_MINIMIZE_ON_FOCUS_LOSS, "0");
+		else if (!GE::getGEConfig()->m_vulkan_fullscreen_desktop)
+			SDL_SetHint(SDL_HINT_VIDEO_MINIMIZE_ON_FOCUS_LOSS, "0");
+	}
 #endif
 
 	u32 init_flags = SDL_INIT_TIMER | SDL_INIT_VIDEO;
@@ -149,33 +159,12 @@ CIrrDeviceSDL::CIrrDeviceSDL(const SIrrlichtCreationParameters& param)
 		}
 		else
 			return;
+		updateNativeScale(&Width, &Height);
+		Width = (u32)((f32)Width * NativeScaleX);
+		Height = (u32)((f32)Height * NativeScaleY);
+		CreationParams.WindowSize.Width = Width;
+		CreationParams.WindowSize.Height = Height;
 	}
-#if !defined(ANDROID) && !defined(__SWITCH__)
-	else if (!GUIEngine::isNoGraphics())
-	{
-		// Get highdpi native scale using renderer so it will work with any
-		// backend later (opengl or vulkan)
-		// Android doesn't use high dpi
-		SDL_Window* window = NULL;
-		SDL_Renderer* renderer = NULL;
-		if (SDL_CreateWindowAndRenderer(640, 480,
-			SDL_WINDOW_ALLOW_HIGHDPI | SDL_WINDOW_HIDDEN,
-			&window, &renderer) == 0)
-		{
-			int w, h, rw, rh;
-			w = h = rw = rh = 0;
-			SDL_GetWindowSize(window, &w, &h);
-			SDL_GetRendererOutputSize(renderer, &rw, &rh);
-			if (w != 0 && h != 0 && rw != 0 && rh != 0)
-			{
-				g_native_scale_x = (float)rw / (float)w;
-				g_native_scale_y = (float)rh / (float)h;
-			}
-			SDL_DestroyRenderer(renderer);
-			SDL_DestroyWindow(window);
-		}
-	}
-#endif
 
 	// create cursor control
 	CursorControl = new CCursorControl(this);
@@ -184,13 +173,42 @@ CIrrDeviceSDL::CIrrDeviceSDL(const SIrrlichtCreationParameters& param)
 	createDriver();
 
 	if (VideoDriver)
-		createGUIAndScene();
+	{
+		if (CreationParams.DriverType == video::EDT_VULKAN)
+			createGUIAndVulkanScene();
+		else
+			createGUIAndScene();
+	}
 #ifdef IOS_STK
 	SDL_SetEventFilter(handle_app_event, NULL);
 #endif
 }
 
 
+void CIrrDeviceSDL::updateNativeScale(u32* saving_width, u32* saving_height)
+{
+	int width, height = 0;
+	SDL_GetWindowSize(Window, &width, &height);
+	int real_width = width;
+	int real_height = height;
+	if (CreationParams.DriverType == video::EDT_OPENGL ||
+		CreationParams.DriverType == video::EDT_OGLES2)
+	{
+		SDL_GL_GetDrawableSize(Window, &real_width, &real_height);
+	}
+	else if (CreationParams.DriverType == video::EDT_VULKAN)
+	{
+		SDL_Vulkan_GetDrawableSize(Window, &real_width, &real_height);
+	}
+	NativeScaleX = (f32)real_width / (f32)width;
+	NativeScaleY = (f32)real_height / (f32)height;
+	if (saving_width)
+		*saving_width = width;
+	if (saving_height)
+		*saving_height = height;
+}
+
+
 //! destructor
 CIrrDeviceSDL::~CIrrDeviceSDL()
 {
@@ -202,6 +220,9 @@ CIrrDeviceSDL::~CIrrDeviceSDL()
 		if (h)
 			h->clearGLExtensions();
 #endif
+		GE::GEVulkanDriver* gevk = dynamic_cast<GE::GEVulkanDriver*>(VideoDriver);
+		if (gevk)
+			gevk->destroyVulkan();
 		VideoDriver->drop();
 		VideoDriver = NULL;
 	}
@@ -315,6 +336,29 @@ bool versionCorrect(int major, int minor)
 }
 
 
+// Used in OptionsScreenVideo for live fullscreen toggle for vulkan driver
+extern "C" void update_fullscreen_desktop(int val)
+{
+	GE::GEVulkanDriver* gevk = GE::getVKDriver();
+	if (!gevk || !GE::getGEConfig()->m_vulkan_fullscreen_desktop)
+		return;
+	SDL_Window* window = gevk->getSDLWindow();
+
+	int prev_width = 0;
+	int prev_height = 0;
+	SDL_GetWindowSize(window, &prev_width, &prev_height);
+
+	if (val != 0)
+		val = SDL_WINDOW_FULLSCREEN_DESKTOP;
+	SDL_SetWindowFullscreen(window, val);
+	if (val == 0)
+	{
+		SDL_SetWindowSize(window, prev_width * 0.8f, prev_height * 0.8f);
+		SDL_RaiseWindow(window);
+	}
+}
+
+
 // Used in OptionsScreenVideo for live updating vertical sync config
 extern "C" void update_swap_interval(int swap_interval)
 {
@@ -323,6 +367,13 @@ extern "C" void update_swap_interval(int swap_interval)
 	if (swap_interval > 1)
 		swap_interval = 1;
 
+	GE::GEVulkanDriver* gevk = GE::getVKDriver();
+	if (gevk)
+	{
+		gevk->updateSwapInterval(swap_interval);
+		return;
+	}
+
 	// Try adaptive vsync first if support
 	if (swap_interval > 0)
 	{
@@ -365,9 +416,25 @@ bool CIrrDeviceSDL::createWindow()
 		}
 	}
 
-	u32 flags = SDL_WINDOW_SHOWN | SDL_WINDOW_ALLOW_HIGHDPI;
+	u32 flags = SDL_WINDOW_SHOWN;
+#if !defined(ANDROID) && !defined(__SWITCH__)
+	if (CreationParams.DriverType == video::EDT_OPENGL ||
+		CreationParams.DriverType == video::EDT_OGLES2 ||
+		CreationParams.DriverType == video::EDT_VULKAN)
+		flags |= SDL_WINDOW_ALLOW_HIGHDPI;
+#endif
+
 	if (CreationParams.Fullscreen)
-		flags |= SDL_WINDOW_FULLSCREEN;
+	{
+		if (CreationParams.DriverType == video::EDT_VULKAN &&
+			GE::getGEConfig()->m_vulkan_fullscreen_desktop)
+		{
+			flags |= SDL_WINDOW_FULLSCREEN_DESKTOP;
+			CreationParams.Fullscreen = false;
+		}
+		else
+			flags |= SDL_WINDOW_FULLSCREEN;
+	}
 
 	if (CreationParams.DriverType == video::EDT_OPENGL ||
 		CreationParams.DriverType == video::EDT_OGLES2)
@@ -381,6 +448,9 @@ bool CIrrDeviceSDL::createWindow()
 			os::Printer::log("Current MacOSX version doesn't support Vulkan or MoltenVK failed to load", ELL_WARNING);
 			return false;
 		}
+#endif
+#if SDL_VERSION_ATLEAST(2, 0, 12)
+		SDL_SetHint(SDL_HINT_VIDEO_EXTERNAL_CONTEXT, "1");
 #endif
 		flags |= SDL_WINDOW_VULKAN;
 	}
@@ -403,13 +473,17 @@ bool CIrrDeviceSDL::createWindow()
 	else
 	{
 		Window = SDL_CreateWindow("",
-			(float)CreationParams.WindowPosition.X / g_native_scale_x,
-			(float)CreationParams.WindowPosition.Y / g_native_scale_y,
-			(float)CreationParams.WindowSize.Width / g_native_scale_x,
-			(float)CreationParams.WindowSize.Height / g_native_scale_y, flags);
+			(float)CreationParams.WindowPosition.X,
+			(float)CreationParams.WindowPosition.Y,
+			(float)CreationParams.WindowSize.Width,
+			(float)CreationParams.WindowSize.Height, flags);
 		if (!Window)
 		{
 			os::Printer::log( "Could not initialize display!" );
+#if SDL_VERSION_ATLEAST(2, 0, 12)
+			if (CreationParams.DriverType == video::EDT_VULKAN)
+				SDL_SetHint(SDL_HINT_VIDEO_EXTERNAL_CONTEXT, "0");
+#endif
 			return false;
 		}
 	}
@@ -449,10 +523,10 @@ start:
 	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 3);
 	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 0);
 	Window = SDL_CreateWindow("",
-		(float)CreationParams.WindowPosition.X / g_native_scale_x,
-		(float)CreationParams.WindowPosition.Y / g_native_scale_y,
-		(float)CreationParams.WindowSize.Width / g_native_scale_x,
-		(float)CreationParams.WindowSize.Height / g_native_scale_y, flags);
+		(float)CreationParams.WindowPosition.X,
+		(float)CreationParams.WindowPosition.Y,
+		(float)CreationParams.WindowSize.Width,
+		(float)CreationParams.WindowSize.Height, flags);
 	if (Window)
 	{
 		Context = SDL_GL_CreateContext(Window);
@@ -475,10 +549,10 @@ start:
 	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 4);
 	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 3);
 	Window = SDL_CreateWindow("",
-		(float)CreationParams.WindowPosition.X / g_native_scale_x,
-		(float)CreationParams.WindowPosition.Y / g_native_scale_y,
-		(float)CreationParams.WindowSize.Width / g_native_scale_x,
-		(float)CreationParams.WindowSize.Height / g_native_scale_y, flags);
+		(float)CreationParams.WindowPosition.X,
+		(float)CreationParams.WindowPosition.Y,
+		(float)CreationParams.WindowSize.Width,
+		(float)CreationParams.WindowSize.Height, flags);
 	if (Window)
 	{
 		Context = SDL_GL_CreateContext(Window);
@@ -500,10 +574,10 @@ start:
 	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 3);
 	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 3);
 	Window = SDL_CreateWindow("",
-		(float)CreationParams.WindowPosition.X / g_native_scale_x,
-		(float)CreationParams.WindowPosition.Y / g_native_scale_y,
-		(float)CreationParams.WindowSize.Width / g_native_scale_x,
-		(float)CreationParams.WindowSize.Height / g_native_scale_y, flags);
+		(float)CreationParams.WindowPosition.X,
+		(float)CreationParams.WindowPosition.Y,
+		(float)CreationParams.WindowSize.Width,
+		(float)CreationParams.WindowSize.Height, flags);
 	if (Window)
 	{
 		Context = SDL_GL_CreateContext(Window);
@@ -525,10 +599,10 @@ start:
 	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 3);
 	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 1);
 	Window = SDL_CreateWindow("",
-		(float)CreationParams.WindowPosition.X / g_native_scale_x,
-		(float)CreationParams.WindowPosition.Y / g_native_scale_y,
-		(float)CreationParams.WindowSize.Width / g_native_scale_x,
-		(float)CreationParams.WindowSize.Height / g_native_scale_y, flags);
+		(float)CreationParams.WindowPosition.X,
+		(float)CreationParams.WindowPosition.Y,
+		(float)CreationParams.WindowSize.Width,
+		(float)CreationParams.WindowSize.Height, flags);
 	if (Window)
 	{
 		Context = SDL_GL_CreateContext(Window);
@@ -562,10 +636,10 @@ legacy:
 	else
 		SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, 0);
 	Window = SDL_CreateWindow("",
-		(float)CreationParams.WindowPosition.X / g_native_scale_x,
-		(float)CreationParams.WindowPosition.Y / g_native_scale_y,
-		(float)CreationParams.WindowSize.Width / g_native_scale_x,
-		(float)CreationParams.WindowSize.Height / g_native_scale_y, flags);
+		(float)CreationParams.WindowPosition.X,
+		(float)CreationParams.WindowPosition.Y,
+		(float)CreationParams.WindowSize.Width,
+		(float)CreationParams.WindowSize.Height, flags);
 	if (Window)
 	{
 		Context = SDL_GL_CreateContext(Window);
@@ -615,10 +689,17 @@ void CIrrDeviceSDL::createDriver()
 		#ifdef _IRR_COMPILE_WITH_VULKAN_
 		try
 		{
-			VideoDriver = video::createVulkanDriver(CreationParams, FileSystem, Window);
+			VideoDriver = video::createVulkanDriver(CreationParams, FileSystem, Window, this);
+			// SDL_Vulkan_GetDrawableSize only works after driver is created
+			updateNativeScale(&Width, &Height);
+			Width = (u32)((f32)Width * NativeScaleX);
+			Height = (u32)((f32)Height * NativeScaleY);
 		}
 		catch (std::exception& e)
 		{
+#if SDL_VERSION_ATLEAST(2, 0, 12)
+			SDL_SetHint(SDL_HINT_VIDEO_EXTERNAL_CONTEXT, "0");
+#endif
 			os::Printer::log("createVulkanDriver failed", e.what(), ELL_ERROR);
 		}
 		#else
@@ -672,6 +753,16 @@ bool CIrrDeviceSDL::run()
 		case SDL_APP_DIDENTERFOREGROUND:
 			resume_mainloop();
 			break;
+#ifdef ANDROID
+		// From https://github.com/libsdl-org/SDL/blob/main/docs/README-android.md
+		// However, there's a chance (on older hardware, or on systems under heavy load),
+		// where the GL context can not be restored. In that case you have to
+		// listen for a specific message (SDL_RENDER_DEVICE_RESET) and restore
+		// your textures manually or quit the app.
+		case SDL_RENDER_DEVICE_RESET:
+			Close = true;
+			break;
+#endif
 #endif
 #if SDL_VERSION_ATLEAST(2, 0, 9)
 		case SDL_SENSORUPDATE:
@@ -741,8 +832,8 @@ bool CIrrDeviceSDL::run()
 		case SDL_MOUSEMOTION:
 			irrevent.EventType = irr::EET_MOUSE_INPUT_EVENT;
 			irrevent.MouseInput.Event = irr::EMIE_MOUSE_MOVED;
-			MouseX = irrevent.MouseInput.X = SDL_event.motion.x * g_native_scale_x;
-			MouseY = irrevent.MouseInput.Y = SDL_event.motion.y * g_native_scale_y;
+			MouseX = irrevent.MouseInput.X = SDL_event.motion.x * NativeScaleX;
+			MouseY = irrevent.MouseInput.Y = SDL_event.motion.y * NativeScaleY;
 			irrevent.MouseInput.ButtonStates = MouseButtonStates;
 
 			postEventFromUser(irrevent);
@@ -752,8 +843,8 @@ bool CIrrDeviceSDL::run()
 		case SDL_MOUSEBUTTONUP:
 
 			irrevent.EventType = irr::EET_MOUSE_INPUT_EVENT;
-			irrevent.MouseInput.X = SDL_event.button.x * g_native_scale_x;
-			irrevent.MouseInput.Y = SDL_event.button.y * g_native_scale_y;
+			irrevent.MouseInput.X = SDL_event.button.x * NativeScaleX;
+			irrevent.MouseInput.Y = SDL_event.button.y * NativeScaleY;
 
 			irrevent.MouseInput.Event = irr::EMIE_MOUSE_MOVED;
 
@@ -859,16 +950,9 @@ bool CIrrDeviceSDL::run()
 
 		case SDL_WINDOWEVENT:
 			{
-				u32 new_width = SDL_event.window.data1 * g_native_scale_x;
-				u32 new_height = SDL_event.window.data2 * g_native_scale_y;
-				if (SDL_event.window.event == SDL_WINDOWEVENT_SIZE_CHANGED &&
-					((new_width != Width) || (new_height != Height)))
+				if (SDL_event.window.event == SDL_WINDOWEVENT_SIZE_CHANGED)
 				{
-					Width = new_width;
-					Height = new_height;
-					if (VideoDriver)
-						VideoDriver->OnResize(core::dimension2d<u32>(Width, Height));
-					reset_network_body();
+					handleNewSize(SDL_event.window.data1, SDL_event.window.data2);
 				}
 				else if (SDL_event.window.event == SDL_WINDOWEVENT_MINIMIZED)
 				{
@@ -882,6 +966,10 @@ bool CIrrDeviceSDL::run()
 				{
 					WindowHasFocus = true;
 					reset_network_body();
+#ifdef ANDROID
+					if (VideoDriver)
+						VideoDriver->unpauseRendering();
+#endif
 				}
 				else if (SDL_event.window.event == SDL_WINDOWEVENT_FOCUS_LOST)
 				{
@@ -930,6 +1018,23 @@ bool CIrrDeviceSDL::run()
 	return !Close;
 }
 
+
+void CIrrDeviceSDL::handleNewSize(u32 width, u32 height)
+{
+	updateNativeScale();
+	u32 new_width = width * NativeScaleX;
+	u32 new_height = height * NativeScaleY;
+	if (new_width != Width || new_height != Height)
+	{
+		Width = new_width;
+		Height = new_height;
+		if (VideoDriver)
+			VideoDriver->OnResize(core::dimension2d<u32>(Width, Height));
+		reset_network_body();
+	}
+}
+
+
 //! Activate any joysticks, and generate events for them.
 bool CIrrDeviceSDL::activateJoysticks(core::array<SJoystickInfo> & joystickInfo)
 {
@@ -1005,20 +1110,20 @@ video::IVideoModeList* CIrrDeviceSDL::getVideoModeList()
 		if (SDL_GetDesktopDisplayMode(0, &mode) == 0)
 		{
 			VideoModeList.setDesktop(SDL_BITSPERPIXEL(mode.format),
-				core::dimension2d<u32>(mode.w * g_native_scale_x, mode.h * g_native_scale_y));
+				core::dimension2d<u32>(mode.w, mode.h));
 		}
 
 #ifdef MOBILE_STK
 	// SDL2 will return w,h and h,w for mobile STK, as we only use landscape
 	// so we just use desktop resolution for now
-	VideoModeList.addMode(core::dimension2d<u32>(mode.w * g_native_scale_x, mode.h * g_native_scale_y),
+	VideoModeList.addMode(core::dimension2d<u32>(mode.w, mode.h),
 		SDL_BITSPERPIXEL(mode.format));
 #else
 		for (int i = 0; i < mode_count; i++)
 		{
 			if (SDL_GetDisplayMode(0, i, &mode) == 0)
 			{
-				VideoModeList.addMode(core::dimension2d<u32>(mode.w * g_native_scale_x, mode.h * g_native_scale_y),
+				VideoModeList.addMode(core::dimension2d<u32>(mode.w, mode.h),
 					SDL_BITSPERPIXEL(mode.format));
 			}
 		}
@@ -1453,7 +1558,7 @@ extern "C" int Android_getMovedHeight();
 s32 CIrrDeviceSDL::getMovedHeight() const
 {
 #if defined(IOS_STK)
-	return SDL_GetMovedHeightByScreenKeyboard() * g_native_scale_y;
+	return SDL_GetMovedHeightByScreenKeyboard() * NativeScaleY;
 #elif defined(ANDROID)
 	return Android_getMovedHeight();
 #else
@@ -1466,7 +1571,7 @@ extern "C" int Android_getKeyboardHeight();
 u32 CIrrDeviceSDL::getOnScreenKeyboardHeight() const
 {
 #if defined(IOS_STK)
-	return SDL_GetScreenKeyboardHeight() * g_native_scale_y;
+	return SDL_GetScreenKeyboardHeight() * NativeScaleY;
 #elif defined(ANDROID)
 	return Android_getKeyboardHeight();
 #else
@@ -1477,13 +1582,13 @@ u32 CIrrDeviceSDL::getOnScreenKeyboardHeight() const
 
 f32 CIrrDeviceSDL::getNativeScaleX() const
 {
-	return g_native_scale_x;
+	return NativeScaleX;
 }
 
 
 f32 CIrrDeviceSDL::getNativeScaleY() const
 {
-	return g_native_scale_y;
+	return NativeScaleY;
 }
 
 
@@ -1543,6 +1648,20 @@ s32 CIrrDeviceSDL::getRightPadding()
 }
 
 
+void CIrrDeviceSDL::createGUIAndVulkanScene()
+{
+	#ifdef _IRR_COMPILE_WITH_GUI_
+	// create gui environment
+	GUIEnvironment = gui::createGUIEnvironment(FileSystem, VideoDriver, Operator);
+	#endif
+
+	// create Scene manager
+	SceneManager = new GE::GEVulkanSceneManager(VideoDriver, FileSystem, CursorControl, GUIEnvironment);
+
+	setEventReceiver(UserReceiver);
+}
+
+
 } // end namespace irr
 
 #endif // _IRR_COMPILE_WITH_SDL_DEVICE_
