@@ -22,8 +22,8 @@
 #include "audio/sfx_buffer.hpp"
 #include "config/user_config.hpp"
 #include "io/file_manager.hpp"
-#include "modes/world.hpp"
 #include "race/race_manager.hpp"
+#include "states_screens/state_manager.hpp"
 #include "utils/stk_process.hpp"
 #include "utils/profiler.hpp"
 #include "utils/string_utils.hpp"
@@ -290,7 +290,7 @@ void SFXManager::queueCommand(SFXCommand *command)
         return;
         
     m_sfx_commands.lock();
-    if(World::getWorld() && 
+    if (StateManager::get()->getGameState() != GUIEngine::MENU &&
         m_sfx_commands.getData().size() > 20*RaceManager::get()->getNumberOfKarts()+20 &&
         RaceManager::get()->getMinorMode() != RaceManager::MINOR_MODE_CUTSCENE)
     {
@@ -392,6 +392,7 @@ void SFXManager::mainLoop(void *obj)
             break;
 #endif
         }
+        bool empty_queue = me->m_sfx_commands.getData().empty();
         ul.unlock();
         PROFILER_POP_CPU_MARKER();
         PROFILER_PUSH_CPU_MARKER("Execute", 0, 255, 0);
@@ -462,9 +463,7 @@ void SFXManager::mainLoop(void *obj)
         current = NULL;
         PROFILER_POP_CPU_MARKER();
         PROFILER_PUSH_CPU_MARKER("yield", 0, 0, 255);
-        // We access the size without lock, doesn't matter if we
-        // should get an incorrect value because of concurrent read/writes
-        if (me->m_sfx_commands.getData().size() == 0 && me->sfxAllowed())
+        if (empty_queue && me->sfxAllowed())
         {
             // Wait some time to let other threads run, then queue an
             // update event to keep music playing.
