@@ -434,12 +434,21 @@ namespace UserConfigParams
     PARAM_PREFIX IntUserConfigParam          m_num_laps
             PARAM_DEFAULT(  IntUserConfigParam(4, "numlaps",
             &m_race_setup_group, "Default number of laps.") );
+    PARAM_PREFIX IntUserConfigParam          m_gp_reverse
+            PARAM_DEFAULT(  IntUserConfigParam(0, "gp-reverse",
+            &m_race_setup_group, "Default direction of GP tracks. 0=default, 1=no reverse, 2=all reverse, 3=Random") );
+    PARAM_PREFIX IntUserConfigParam          m_rand_gp_num_tracks
+            PARAM_DEFAULT(  IntUserConfigParam(1, "random-gp-num-tracks",
+            &m_race_setup_group, "Default number of tracks for random GP.") );            
     PARAM_PREFIX IntUserConfigParam          m_ffa_time_limit
         PARAM_DEFAULT(IntUserConfigParam(3, "ffa-time-limit",
             &m_race_setup_group, "Time limit in ffa mode."));
     PARAM_PREFIX BoolUserConfigParam         m_use_ffa_mode
         PARAM_DEFAULT(BoolUserConfigParam(false, "use-ffa-mode",
             &m_race_setup_group, "Use ffa mode instead of 3 strikes battle."));
+    PARAM_PREFIX IntUserConfigParam          m_lap_trial_time_limit
+        PARAM_DEFAULT(IntUserConfigParam(3, "lap-trial-time-limit",
+            &m_race_setup_group, "Time limit in lap trial mode."));
     PARAM_PREFIX IntUserConfigParam          m_num_goals
             PARAM_DEFAULT(  IntUserConfigParam(3, "numgoals",
             &m_race_setup_group, "Default number of goals in soccer mode.") );
@@ -486,6 +495,9 @@ namespace UserConfigParams
     PARAM_PREFIX BoolUserConfigParam          m_addon_tux_online
             PARAM_DEFAULT(  BoolUserConfigParam(false, "addon-tux-online",
             &m_race_setup_group, "Always show online addon karts as tux when live join is on.") );
+    PARAM_PREFIX BoolUserConfigParam          m_random_player_pos
+            PARAM_DEFAULT(  BoolUserConfigParam(false, "random-player-pos",
+            &m_race_setup_group, "Randomize the position of the players at the start of a race. Doesn't apply to story mode.") );
 
     // ---- Wiimote data
     PARAM_PREFIX GroupUserConfigParam        m_wiimote_group
@@ -599,21 +611,27 @@ namespace UserConfigParams
     PARAM_PREFIX GroupUserConfigParam        m_video_group
         PARAM_DEFAULT( GroupUserConfigParam("Video", "Video Settings") );
 
+    PARAM_PREFIX IntUserConfigParam         m_real_width
+            PARAM_DEFAULT(  IntUserConfigParam(1024, "real_width", &m_video_group,
+                                            "Screen/window real width in pixels before high dpi is applied") );
+    PARAM_PREFIX IntUserConfigParam         m_real_height
+            PARAM_DEFAULT(  IntUserConfigParam(768, "real_height", &m_video_group,
+                                           "Screen/window real height in pixels before high dpi is applied") );
     PARAM_PREFIX IntUserConfigParam         m_width
             PARAM_DEFAULT(  IntUserConfigParam(1024, "width", &m_video_group,
-                                            "Screen/window width in pixels") );
+                                            "Screen/window width in pixels, this value should not be edited") );
     PARAM_PREFIX IntUserConfigParam         m_height
             PARAM_DEFAULT(  IntUserConfigParam(768, "height", &m_video_group,
-                                           "Screen/window height in pixels") );
+                                           "Screen/window height in pixels, this value should not be edited") );
     PARAM_PREFIX BoolUserConfigParam        m_fullscreen
             PARAM_DEFAULT(  BoolUserConfigParam(false, "fullscreen",
                                                 &m_video_group) );
-    PARAM_PREFIX IntUserConfigParam         m_prev_width
-            PARAM_DEFAULT(  IntUserConfigParam(1024, "prev_width",
-                            &m_video_group, "Previous screen/window width") );
-    PARAM_PREFIX IntUserConfigParam         m_prev_height
-            PARAM_DEFAULT(  IntUserConfigParam(768, "prev_height",
-                            &m_video_group,"Previous screen/window height") );
+    PARAM_PREFIX IntUserConfigParam         m_prev_real_width
+            PARAM_DEFAULT(  IntUserConfigParam(1024, "prev_real_width",
+                            &m_video_group, "Previous real screen/window width") );
+    PARAM_PREFIX IntUserConfigParam         m_prev_real_height
+            PARAM_DEFAULT(  IntUserConfigParam(768, "prev_real_height",
+                            &m_video_group,"Previous real screen/window height") );
     PARAM_PREFIX BoolUserConfigParam        m_prev_fullscreen
             PARAM_DEFAULT(  BoolUserConfigParam(false, "prev_fullscreen",
                             &m_video_group) );
@@ -694,14 +712,24 @@ namespace UserConfigParams
         PARAM_DEFAULT(  FloatUserConfigParam(3, "font_size",
         &m_video_group, "The size of fonts. 0 is the smallest and 6 is the biggest") );
 
-#if defined(_IRR_COMPILE_WITH_DIRECT3D_9_) && defined(_M_ARM64)
+#if defined(_IRR_COMPILE_WITH_DIRECT3D_9_) && defined(_M_ARM)
     PARAM_PREFIX StringUserConfigParam         m_render_driver
         PARAM_DEFAULT(  StringUserConfigParam("directx9", "render_driver",
-        &m_video_group, "Render video driver to use, at the moment gl or directx9 is supported.") );
+        &m_video_group, "Render video driver to use, at the moment gl, vulkan or directx9 is supported.") );
 #else
     PARAM_PREFIX StringUserConfigParam         m_render_driver
         PARAM_DEFAULT(  StringUserConfigParam("gl", "render_driver",
-        &m_video_group, "Render video driver to use, at the moment gl or directx9 is supported.") );
+        &m_video_group, "Render video driver to use, at the moment gl, vulkan or directx9 is supported.") );
+#endif
+
+#if defined(MOBILE_STK)
+    PARAM_PREFIX BoolUserConfigParam        m_vulkan_fullscreen_desktop
+        PARAM_DEFAULT(BoolUserConfigParam(false, "vulkan_fullscreen_desktop",
+        &m_video_group, "Use SDL_WINDOW_FULLSCREEN_DESKTOP for vulkan device"));
+#else
+    PARAM_PREFIX BoolUserConfigParam        m_vulkan_fullscreen_desktop
+        PARAM_DEFAULT(BoolUserConfigParam(true, "vulkan_fullscreen_desktop",
+        &m_video_group, "Use SDL_WINDOW_FULLSCREEN_DESKTOP for vulkan device"));
 #endif
 
     // ---- Recording
@@ -802,6 +830,10 @@ namespace UserConfigParams
     /** True if graphical profiler should be displayed */
     PARAM_PREFIX bool m_profiler_enabled  PARAM_DEFAULT( false );
 
+    PARAM_PREFIX bool m_disable_addon_karts  PARAM_DEFAULT( false );
+
+    PARAM_PREFIX bool m_disable_addon_tracks  PARAM_DEFAULT( false );
+
     // ---- Networking
     PARAM_PREFIX StringToUIntUserConfigParam    m_server_bookmarks
         PARAM_DEFAULT(StringToUIntUserConfigParam("server-bookmarks",
@@ -1195,9 +1227,38 @@ namespace UserConfigParams
             PARAM_DEFAULT(  StringUserConfigParam("peach", "skin_name",
                                                   "Name of the skin to use") );
 
+    // ---- settings for minimap display
+    PARAM_PREFIX GroupUserConfigParam        m_minimap_setup_group
+        PARAM_DEFAULT( GroupUserConfigParam("Minimap",
+                                            "Minimap Setup Settings") );
+
     PARAM_PREFIX IntUserConfigParam        m_minimap_display
-        PARAM_DEFAULT(IntUserConfigParam(0, "minimap_display",
-                      "Minimap: 0 bottom-left, 1 middle-right, 2 hidden, 3 center"));
+        PARAM_DEFAULT(IntUserConfigParam(0, "display",
+                     &m_minimap_setup_group, "display: 0 bottom-left, 1 middle-right, 2 hidden, 3 center"));
+
+    PARAM_PREFIX FloatUserConfigParam      m_minimap_size
+            PARAM_DEFAULT(  FloatUserConfigParam(180.0f, "size",
+            &m_minimap_setup_group, "Size of the the minimap (480 = full screen height; scaled afterwards)") );
+
+    PARAM_PREFIX FloatUserConfigParam      m_minimap_ai_icon_size
+            PARAM_DEFAULT(  FloatUserConfigParam(16.0f, "ai-icon",
+            &m_minimap_setup_group, "The size of the icons for the AI karts on the minimap.") );
+
+    PARAM_PREFIX FloatUserConfigParam      m_minimap_player_icon_size
+            PARAM_DEFAULT(  FloatUserConfigParam(20.0f, "player-icon",
+            &m_minimap_setup_group, "The size of the icons for the player kart.") );
+
+    // ---- settings for powerup display
+    PARAM_PREFIX GroupUserConfigParam      m_powerup_setup_group
+        PARAM_DEFAULT( GroupUserConfigParam("PowerUp",
+                                            "PowerUp Setup Settings") );
+
+    PARAM_PREFIX IntUserConfigParam        m_powerup_display
+        PARAM_DEFAULT(IntUserConfigParam(0, "display",
+            &m_powerup_setup_group, "display: 0 center, 1 right side, 2 hidden (see karts' held powerups)"));
+    PARAM_PREFIX FloatUserConfigParam      m_powerup_size
+            PARAM_DEFAULT(  FloatUserConfigParam(64.0f, "powerup-icon-size",
+            &m_powerup_setup_group, "Size of the powerup icon (scaled afterwards)") );
 
     // ---- Settings for spectator camera
     PARAM_PREFIX GroupUserConfigParam       m_spectator
