@@ -21,6 +21,8 @@
 #include "config/stk_config.hpp"
 #include "config/user_config.hpp"
 #include "graphics/irr_driver.hpp"
+#include "graphics/material.hpp"
+#include "graphics/material_manager.hpp"
 #include "graphics/sp/sp_base.hpp"
 #include "io/file_manager.hpp"
 #include "karts/abstract_kart.hpp"
@@ -47,6 +49,7 @@
 std::vector<scene::IMesh *>  ItemManager::m_item_mesh;
 std::vector<scene::IMesh *>  ItemManager::m_item_lowres_mesh;
 std::vector<video::SColorf>  ItemManager::m_glow_color;
+std::vector<std::string>     ItemManager::m_icon;
 bool                         ItemManager::m_disable_item_collection = false;
 std::mt19937                 ItemManager::m_random_engine;
 uint32_t                     ItemManager::m_random_seed = 0;
@@ -59,9 +62,11 @@ void ItemManager::loadDefaultItemMeshes()
     m_item_mesh.clear();
     m_item_lowres_mesh.clear();
     m_glow_color.clear();
+    m_icon.clear();
     m_item_mesh.resize(ItemState::ITEM_LAST-ItemState::ITEM_FIRST+1, NULL);
     m_glow_color.resize(ItemState::ITEM_LAST-ItemState::ITEM_FIRST+1,
                         video::SColorf(255.0f, 255.0f, 255.0f) );
+    m_icon.resize(ItemState::ITEM_LAST-ItemState::ITEM_FIRST+1, "");
 
     m_item_lowres_mesh.resize(ItemState::ITEM_LAST-ItemState::ITEM_FIRST+1, NULL);
 
@@ -113,10 +118,26 @@ void ItemManager::loadDefaultItemMeshes()
 #endif
             m_item_lowres_mesh[i]->grab();
         }
+        m_icon[i] = "icon-" + item_names[(ItemState::ItemType)i] + ".png";
+        preloadIcon(m_icon[i]);
     }   // for i
     delete root;
+    preloadIcon("item_spark.png");
 }   // loadDefaultItemMeshes
 
+//-----------------------------------------------------------------------------
+/** Preload icon materials to avoid hangs when firstly insert item
+ */
+void ItemManager::preloadIcon(const std::string& name)
+{
+    // From IrrDriver::addBillboard
+    Material* m = material_manager->getMaterial(name, false/*full_path*/,
+        /*make_permanent*/true, /*complain_if_not_found*/true,
+        /*strip_path*/false, /*install*/false);
+    m->getTexture(true/*srgb*/, m->getShaderName() == "additive" ||
+        m->getShaderName() == "alphablend" ? true : false/*premul_alpha*/);
+}   // preloadIcon
+
 //-----------------------------------------------------------------------------
 /** Clean up all textures. This is necessary when switching resolution etc.
  */
@@ -297,7 +318,8 @@ Item* ItemManager::dropNewItem(ItemState::ItemType type,
     }
 
     Item* item = new Item(type, pos, normal, m_item_mesh[mesh_type],
-                          m_item_lowres_mesh[mesh_type], /*prev_owner*/kart);
+                          m_item_lowres_mesh[mesh_type], m_icon[mesh_type],
+                          /*prev_owner*/kart);
 
     // restoreState in NetworkItemManager will handle the insert item
     if (!server_xyz)
@@ -329,7 +351,8 @@ Item* ItemManager::placeItem(ItemState::ItemType type, const Vec3& xyz,
     ItemState::ItemType mesh_type = type;
 
     Item* item = new Item(type, xyz, normal, m_item_mesh[mesh_type],
-                          m_item_lowres_mesh[mesh_type], /*prev_owner*/NULL);
+                          m_item_lowres_mesh[mesh_type], m_icon[mesh_type],
+                          /*prev_owner*/NULL);
 
     insertItem(item);
     if (m_switch_ticks >= 0)
