@@ -31,7 +31,6 @@
 #include "graphics/material.hpp"
 #include "graphics/material_manager.hpp"
 #include "graphics/mesh_tools.hpp"
-#include "graphics/sp/sp_animation.hpp"
 #include "graphics/sp/sp_mesh.hpp"
 #include "graphics/sp/sp_mesh_buffer.hpp"
 #include "graphics/sp/sp_mesh_node.hpp"
@@ -48,6 +47,9 @@
 
 #include "IMeshManipulator.h"
 #include <algorithm>
+#include <ge_animation.hpp>
+#include <ge_render_info.hpp>
+#include <ge_spm.hpp>
 
 #define SKELETON_DEBUG 0
 
@@ -317,7 +319,7 @@ KartModel::~KartModel()
  *  It is also marked not to be a master copy, so attachModel can be called
  *  for this instance.
  */
-KartModel* KartModel::makeCopy(std::shared_ptr<RenderInfo> ri)
+KartModel* KartModel::makeCopy(std::shared_ptr<GE::GERenderInfo> ri)
 {
     // Make sure that we are copying from a master objects, and
     // that there is indeed no animated node defined here ...
@@ -450,7 +452,6 @@ scene::ISceneNode* KartModel::attachModel(bool animated_models, bool human_playe
         m_wheel_graphics_radius[i] = 0.5f*(wheel_max.getY() - wheel_min.getY());
 
         m_wheel_node[i]->grab();
-        ((scene::IMeshSceneNode *) m_wheel_node[i])->setReadOnlyMaterials(true);
 #ifdef DEBUG
         std::string debug_name = m_wheel_filename[i]+" (wheel)";
         m_wheel_node[i]->setName(debug_name.c_str());
@@ -592,20 +593,23 @@ bool KartModel::loadModels(const KartProperties &kart_properties)
     Vec3 kart_max = m_mesh->getMax();
 #ifndef SERVER_ONLY
     // Test if kart model support colorization
-    if (CVS->isGLSL())
+    for (u32 i = 0; i < m_mesh->getMeshBufferCount(); i++)
     {
-        for (u32 i = 0; i < m_mesh->getMeshBufferCount(); i++)
+        SP::SPMeshBuffer* spmb =
+            dynamic_cast<SP::SPMeshBuffer*>(m_mesh->getMeshBuffer(i));
+        if (!spmb)
         {
-            SP::SPMeshBuffer* mb =
-                static_cast<SP::SPMeshBuffer*>(m_mesh->getMeshBuffer(i));
-            // Pre-upload gl meshes and textures for kart screen
-            mb->uploadGLMesh();
-            std::vector<Material*> mbs = mb->getAllSTKMaterials();
-            for (Material* m : mbs)
-            {
-                m_support_colorization =
-                    m_support_colorization || m->isColorizable();
-            }
+            m_support_colorization = m_support_colorization ||
+                m_mesh->getMeshBuffer(i)->getMaterial().isColorizable();
+            continue;
+        }
+        // Pre-upload gl meshes and textures for kart screen
+        spmb->uploadGLMesh();
+        std::vector<Material*> mbs = spmb->getAllSTKMaterials();
+        for (Material* m : mbs)
+        {
+            m_support_colorization =
+                m_support_colorization || m->isColorizable();
         }
     }
 #endif
@@ -1256,9 +1260,11 @@ void KartModel::resetVisualWheelPosition()
 }   // resetVisualSuspension
 
 //-----------------------------------------------------------------------------
-std::shared_ptr<RenderInfo> KartModel::getRenderInfo()
+std::shared_ptr<GE::GERenderInfo> KartModel::getRenderInfo()
 {
-    return m_support_colorization ? m_render_info : NULL;
+    return m_support_colorization ||
+        (m_render_info && m_render_info->isTransparent()) ?
+        m_render_info : NULL;
 }   // getRenderInfo
 
 //-----------------------------------------------------------------------------
@@ -1295,11 +1301,21 @@ void KartModel::initInverseBoneMatrices()
             m_model_filename.c_str());
         striaght_frame = 0.0f;
     }
+    std::vector<GE::Armature> armatures;
+
+#ifndef SERVER_ONLY
     using namespace SP;
     SPMesh* spm = dynamic_cast<SPMesh*>(m_mesh);
+    GE::GESPM* ge_spm = dynamic_cast<GE::GESPM*>(m_mesh);
     if (spm)
+        armatures = spm->getArmatures();
+    else if (ge_spm)
+        armatures = ge_spm->getArmatures();
+#endif
+
+    if (!armatures.empty())
     {
-        for (Armature& arm : spm->getArmatures())
+        for (GE::Armature& arm : armatures)
         {
             arm.getInterpolatedMatrices(striaght_frame);
             for (auto& p : arm.m_world_matrices)
@@ -1328,6 +1344,8 @@ void KartModel::initInverseBoneMatrices()
             node->setCurrentFrame(striaght_frame);
             node->OnAnimate(0);
             scene::IBoneSceneNode* bone = node->getJointNode(i);
+            if (!bone)
+                continue;
             bone->updateAbsolutePosition();
             node->setCurrentFrame(striaght_frame);
             node->OnAnimate(0);
@@ -1352,8 +1370,11 @@ void KartModel::initInverseBoneMatrices()
 const core::matrix4& KartModel::getInverseBoneMatrix
                                            (const std::string& bone_name) const
 {
+    // Remove after GESPM animation is done
+    static core::matrix4 unused;
     assert(m_version >= 3);
     auto ret = m_inverse_bone_matrices.find(bone_name);
-    assert(ret != m_inverse_bone_matrices.end());
+    if (ret == m_inverse_bone_matrices.end())
+        return unused;
     return ret->second;
 }   // getInverseBoneMatrix
