@@ -29,7 +29,7 @@
 #include "graphics/irr_driver.hpp"
 #include "graphics/material.hpp"
 #include "graphics/material_manager.hpp"
-#include "graphics/render_info.hpp"
+#include <ge_render_info.hpp>
 #include "guiengine/modaldialog.hpp"
 #include "guiengine/screen_keyboard.hpp"
 #include "io/file_manager.hpp"
@@ -322,7 +322,7 @@ void World::initTeamArrows(AbstractKart* k)
 
     KartModel* km = k->getKartModel();
     // Color of karts can be changed using shaders if the model supports
-    if (km->supportColorization() && CVS->isGLSL())
+    if (km->supportColorization() && CVS->supportsColorization())
         return;
 
     float arrow_pos_height = km->getHeight() + 0.5f;
@@ -463,7 +463,7 @@ std::shared_ptr<AbstractKart> World::createKart
     if (RaceManager::get()->hasGhostKarts())
         gk = ReplayPlay::get()->getNumGhostKart();
 
-    std::shared_ptr<RenderInfo> ri = std::make_shared<RenderInfo>();
+    std::shared_ptr<GE::GERenderInfo> ri = std::make_shared<GE::GERenderInfo>();
     core::stringw online_name;
     if (global_player_id > -1)
     {
@@ -1262,7 +1262,7 @@ Highscores* World::getHighscores() const
                                          RaceManager::get()->getNumNonGhostKarts(),
                                          RaceManager::get()->getDifficulty(),
                                          RaceManager::get()->getTrackName(),
-                                         RaceManager::get()->getNumLaps(),
+                                         RaceManager::get()->isLapTrialMode() ? RaceManager::get()->getTimeTarget() : RaceManager::get()->getNumLaps(),
                                          RaceManager::get()->getReverseTrack());
 
     return highscores;
@@ -1274,6 +1274,7 @@ Highscores* World::getGPHighscores() const
     Highscores* highscores = highscore_manager->getGPHighscores(RaceManager::get()->getNumNonGhostKarts(),
                                                                 RaceManager::get()->getDifficulty(),
                                                                 RaceManager::get()->getGrandPrix().getId(),
+                                                                RaceManager::get()->isLapTrialMode() ? RaceManager::get()->getTimeTarget() : 0,
                                                                 RaceManager::get()->getGrandPrix().getReverseType(),
                                                                 RaceManager::get()->getMinorMode());
     return highscores;
@@ -1341,9 +1342,19 @@ void World::updateHighscores(int* best_highscore_rank)
 
         int highscore_rank = 0;
         // The player is a local player, so there is a name:
-        highscore_rank = highscores->addData(k->getIdent(),
-                                             k->getController()->getName(),
-                                             k->getFinishTime()    );
+        if (RaceManager::get()->isLapTrialMode())
+        {
+            highscore_rank = highscores->addData(k->getIdent(),
+                                                 k->getController()->getName(),
+                                                 static_cast<float>(getFinishedLapsOfKart(index[pos])));
+        }
+        else
+        {
+            highscore_rank = highscores->addData(k->getIdent(),
+                                                 k->getController()->getName(),
+                                                 k->getFinishTime()    );
+        }
+        
 
         if (highscore_rank > 0)
         {
@@ -1585,9 +1596,9 @@ std::shared_ptr<AbstractKart> World::createKartWithTeam
     btTransform init_pos = getStartTransform(pos_index - 1);
     m_kart_position_map[index] = (unsigned)(pos_index - 1);
 
-    std::shared_ptr<RenderInfo> ri = std::make_shared<RenderInfo>();
-    ri = (team == KART_TEAM_BLUE ? std::make_shared<RenderInfo>(0.66f) :
-        std::make_shared<RenderInfo>(1.0f));
+    std::shared_ptr<GE::GERenderInfo> ri = std::make_shared<GE::GERenderInfo>();
+    ri = (team == KART_TEAM_BLUE ? std::make_shared<GE::GERenderInfo>(0.66f) :
+        std::make_shared<GE::GERenderInfo>(1.0f));
 
     std::shared_ptr<AbstractKart> new_kart;
     if (RewindManager::get()->isEnabled())
