@@ -20,6 +20,7 @@
 
 #include <iostream>
 #include <algorithm>
+#include <random>
 
 #include "challenges/unlock_manager.hpp"
 #include "config/player_manager.hpp"
@@ -46,6 +47,7 @@
 #include "modes/world.hpp"
 #include "modes/three_strikes_battle.hpp"
 #include "modes/soccer_world.hpp"
+#include "modes/lap_trial.hpp"
 #include "network/protocol_manager.hpp"
 #include "network/network_config.hpp"
 #include "network/network_string.hpp"
@@ -132,6 +134,8 @@ RaceManager::RaceManager()
     m_flag_return_ticks = stk_config->time2Ticks(20.0f);
     m_flag_deactivated_ticks = stk_config->time2Ticks(3.0f);
     m_skipped_tracks_in_gp = 0;
+    m_gp_time_target = 0.0f;
+    m_gp_total_laps = 0;
     setMaxGoal(0);
     setTimeTarget(0.0f);
     setReverseTrack(false);
@@ -421,6 +425,12 @@ void RaceManager::startNew(bool from_overworld)
                                                 m_saved_gp->getReverseType());
                     m_reverse_track = m_grand_prix.getReverse();
                     m_skipped_tracks_in_gp = m_saved_gp->getSkippedTracks();
+                    Log::info("RaceManager","%d",isLapTrialMode());
+                    if (isLapTrialMode())
+                    {
+                        m_gp_time_target = m_saved_gp->getTimeTarget();
+                        m_gp_total_laps = m_saved_gp->getPlayerTotalLaps();
+                    }
                 }   // if m_saved_gp==NULL
             }   // if m_continue_saved_gp
         }   // if !network_world
@@ -439,6 +449,12 @@ void RaceManager::startNew(bool from_overworld)
 
     Log::verbose("RaceManager", "Nb of karts=%u, ghost karts:%u ai:%lu players:%lu\n",
         (unsigned int) m_num_karts, m_num_ghost_karts, m_ai_kart_list.size(), m_player_karts.size());
+    std::set<std::string> used_karts;
+    for (auto& kart : m_ai_kart_list)
+        used_karts.insert(kart);
+    for (auto& kart : m_player_karts)
+        used_karts.insert(kart.getKartName());
+    kart_properties_manager->onDemandLoadKartTextures(used_karts);
 
     assert((unsigned int)m_num_karts == m_num_ghost_karts+m_ai_kart_list.size()+m_player_karts.size());
 
@@ -532,7 +548,7 @@ void RaceManager::startNextRace()
 #ifdef __SWITCH__
     // Throttles GPU while boosting CPU
     appletSetCpuBoostMode(ApmCpuBoostMode_FastLoad);
-#endif
+#endif  
     ProcessType type = STKProcess::getType();
     main_loop->renderGUI(0);
     // Uncomment to debug audio leaks
@@ -546,19 +562,19 @@ void RaceManager::startNextRace()
         GUIEngine::clearLoadingTips();
         GUIEngine::renderLoading(true/*clearIcons*/, false/*launching*/, false/*update_tips*/);
         device->getVideoDriver()->endScene();
+        GUIEngine::flushRenderLoading(false/*launching*/);
     }
 
     m_num_finished_karts   = 0;
     m_num_finished_players = 0;
+    // In follow the leader mode do not change the first kart,
+    // since it's always the leader.
+    int offset = (m_minor_mode==MINOR_MODE_FOLLOW_LEADER) ? 1 : 0;
 
     // if subsequent race, sort kart status structure
     // ==============================================
     if (m_track_number > 0)
     {
-        // In follow the leader mode do not change the first kart,
-        // since it's always the leader.
-        int offset = (m_minor_mode==MINOR_MODE_FOLLOW_LEADER) ? 1 : 0;
-
         // Keep players at the end if needed
         int player_last_offset = 0;
         if (UserConfigParams::m_gp_player_last)
@@ -579,6 +595,23 @@ void RaceManager::startNextRace()
                          m_kart_status.end() - player_last_offset);
         }
     }   // not first race
+    else
+    {
+        const bool random_pos_available = !NetworkConfig::get()->isNetworking() &&
+            (RaceManager::get()->getMinorMode() == RaceManager::MINOR_MODE_NORMAL_RACE
+            || RaceManager::get()->getMinorMode() == RaceManager::MINOR_MODE_TIME_TRIAL
+            || RaceManager::get()->getMinorMode() == RaceManager::MINOR_MODE_FOLLOW_LEADER);
+
+        if (UserConfigParams::m_random_player_pos)
+        {
+            if (random_pos_available)
+            {
+                unsigned seed = std::chrono::system_clock::now().time_since_epoch().count();
+                std::shuffle(m_kart_status.begin() + offset, m_kart_status.end(),
+                    std::default_random_engine(seed));
+            }
+        }
+    }
 
     // set boosted AI status for AI karts
     int boosted_ai_count = std::min<int>((int)m_ai_kart_list.size(),
@@ -620,6 +653,12 @@ void RaceManager::startNextRace()
     else if(m_minor_mode==MINOR_MODE_NORMAL_RACE ||
             m_minor_mode==MINOR_MODE_TIME_TRIAL)
         World::setWorld(new StandardRace());
+    else if(m_minor_mode==MINOR_MODE_LAP_TRIAL)
+    {
+        World::setWorld(new LapTrial());
+        if (m_major_mode == MAJOR_MODE_GRAND_PRIX)
+            RaceManager::get()->setTimeTarget(m_gp_time_target);
+    }
     else if(m_minor_mode==MINOR_MODE_TUTORIAL)
         World::setWorld(new TutorialWorld());
     else if (isBattleMode())
@@ -749,6 +788,8 @@ void RaceManager::saveGP()
             m_track_number,
             m_grand_prix.getReverseType(),
             m_skipped_tracks_in_gp,
+            isLapTrialMode() ? m_gp_time_target : 0.0f,
+            isLapTrialMode() ? m_gp_total_laps : 0,
             m_kart_status);
 
         // If a new GP is saved, delete any other saved data for this
@@ -944,6 +985,7 @@ void RaceManager::exitRace(bool delete_world)
         setNumKarts(0);
         setNumPlayers(0);
 
+        std::set<std::string> used_karts;
         if (some_human_player_well_ranked)
         {
             startSingleRace("gpwin", 999,
@@ -952,6 +994,9 @@ void RaceManager::exitRace(bool delete_world)
             scene->push();
             scene->setKarts(winners);
             scene->setPlayerWon(some_human_player_won);
+            std::set<std::string> karts;
+            for (auto& kart : winners)
+                used_karts.insert(kart.first);
         }
         else
         {
@@ -963,6 +1008,8 @@ void RaceManager::exitRace(bool delete_world)
             if (humanLosers.size() >= 1)
             {
                 scene->setKarts(humanLosers);
+                for (auto& kart : humanLosers)
+                    used_karts.insert(kart.first);
             }
             else
             {
@@ -970,9 +1017,11 @@ void RaceManager::exitRace(bool delete_world)
                            "This should have never happened\n");
                 std::vector<std::pair<std::string, float> > karts;
                 karts.emplace_back(UserConfigParams::m_default_kart, 0.0f);
+                used_karts.insert(UserConfigParams::m_default_kart);
                 scene->setKarts(karts);
             }
         }
+        kart_properties_manager->onDemandLoadKartTextures(used_karts);
     }
 
     if (delete_world)
@@ -982,6 +1031,8 @@ void RaceManager::exitRace(bool delete_world)
         World::deleteWorld();
     }
 
+    // Reload track screenshot after delete_world (track textures are unloaded)
+    track_manager->onDemandLoadTrackScreenshots();
     m_saved_gp = NULL;
     m_track_number = 0;
 
@@ -1232,6 +1283,8 @@ const core::stringw RaceManager::getNameOf(const MinorRaceModeType mode)
         //I18N: Game mode
         case MINOR_MODE_FOLLOW_LEADER:  return _("Follow the Leader");
         //I18N: Game mode
+        case MINOR_MODE_LAP_TRIAL:      return _("Lap Trial");
+        //I18N: Game mode
         case MINOR_MODE_3_STRIKES:      return _("3 Strikes Battle");
         //I18N: Game mode
         case MINOR_MODE_FREE_FOR_ALL:   return _("Free-For-All");
