@@ -118,6 +118,7 @@ private:
     std::string              m_screenshot;
     bool                     m_is_day;
     std::vector<MusicInformation*> m_music;
+    unsigned                       m_music_idx;
 
     /** Will only be used on overworld */
     std::vector<OverworldChallenge> m_challenges;
@@ -193,6 +194,9 @@ private:
     scene::ISceneNode  *m_sun;
     /** Used to collect the triangles for the bullet mesh. */
     TriangleMesh*            m_track_mesh;
+    /** Used to collect the triangles for the height map mesh used for
+     *  particle rendering. */
+    TriangleMesh*            m_height_map_mesh;
     /** Used to collect the triangles which do not have a physical
      *  representation, but are needed for some raycast effects. An
      *  example is a water surface: the karts ignore this (i.e.
@@ -445,7 +449,8 @@ public:
     void               removeCachedData  ();
     void               startMusic        () const;
 
-    void               createPhysicsModel(unsigned int main_track_count);
+    void               createPhysicsModel(unsigned int main_track_count,
+                                          bool for_height_map);
     void               updateGraphics(float dt);
     void               update(int ticks);
     void               reset();
@@ -732,6 +737,13 @@ public:
     // ------------------------------------------------------------------------
     bool isOnGround(const Vec3& xyz, const Vec3& down, Vec3* hit_point,
                     Vec3* normal, bool print_warning = true);
+    // ------------------------------------------------------------------------
+    MusicInformation* getTrackMusic() const
+    {
+        if (m_music_idx < m_music.size())
+            return m_music[m_music_idx];
+        return NULL;
+    }
 };   // class Track
 
 #endif
