@@ -101,13 +101,14 @@
                 getCallStack(callstack);
 
             std::string msg =   "SuperTuxKart crashed!\n"
-                                "Please hit Ctrl+C to copy to clipboard and signal the problem\n"
-                                "to the developers on our forum: http://forum.freegamedev.net/viewforum.php?f=16\n"
+                                "If you continue to encounter this issue, please hit Ctrl+C to copy this error\n"
+                                "to the clipboard and report the problem to the developers on our forum:\n"
+                                "http://forum.freegamedev.net/viewforum.php?f=17\n"
                                 "\n"
                                 "Call stack:\n";
             msg += callstack;
             Log::error("StackTrace", "%s", msg.c_str());
-            MessageBoxA(NULL, msg.c_str(), "SuperTuxKart crashed :/", MB_OK);
+            MessageBoxA(NULL, msg.c_str(), "SuperTuxKart crashed!", MB_ICONERROR | MB_OK);
         }   // winCrashHandler
 
         // --------------------------------------------------------------------
@@ -239,7 +240,12 @@
                 stackframe.AddrPC.Mode      = AddrModeFlat;
                 stackframe.AddrStack.Mode   = AddrModeFlat;
                 stackframe.AddrFrame.Mode   = AddrModeFlat;
-#if defined(_M_ARM64)
+#if defined(_M_ARM)
+                stackframe.AddrPC.Offset    = pContext->Pc;
+                stackframe.AddrStack.Offset = pContext->Sp;
+                stackframe.AddrFrame.Offset = pContext->R11;
+                const DWORD machine_type    = IMAGE_FILE_MACHINE_ARM;
+#elif defined(_M_ARM64)
                 stackframe.AddrPC.Offset    = pContext->Pc;
                 stackframe.AddrStack.Offset = pContext->Sp;
                 stackframe.AddrFrame.Offset = pContext->Fp;
@@ -429,6 +435,8 @@
             path = realpath("/proc/self/exe", NULL);
 #elif defined(__NetBSD__)
             path = realpath("/proc/curproc/exe", NULL);
+#elif defined(__sun)
+	    path = realpath("/proc/self/path/a.out", NULL);
 #elif defined(__APPLE__)
             path = (char*)malloc(PATH_MAX+1);
             uint32_t len = PATH_MAX;
